@@ -1,6 +1,6 @@
 module BinData
-
-  class UnRegisteredTypeError < StandardError ; end
+  # Raised when #lookup fails.
+  class UnRegisteredTypeError < StandardError; end
 
   # This registry contains a register of name -> class mappings.
   #
@@ -18,13 +18,12 @@ module BinData
   #
   # Names are stored in under_score_style, not camelCase.
   class Registry
-
     def initialize
       @registry = {}
     end
 
     def register(name, class_to_register)
-      return if class_to_register.nil?
+      return if name.nil? || class_to_register.nil?
 
       formatted_name = underscore_name(name)
       warn_if_name_is_already_registered(formatted_name, class_to_register)
@@ -37,47 +36,56 @@ module BinData
     end
 
     def lookup(name, hints = {})
-      key = normalize_name(name, hints)
-      @registry[key] || raise(UnRegisteredTypeError, name.to_s)
+      search_names(name, hints).each do |search|
+        register_dynamic_class(search)
+        if @registry.has_key?(search)
+          return @registry[search]
+        end
+      end
+
+      # give the user a hint if the endian keyword is missing
+      search_names(name, hints.merge(endian: :big)).each do |search|
+        register_dynamic_class(search)
+        if @registry.has_key?(search)
+          raise(UnRegisteredTypeError, "#{name}, do you need to specify endian?")
+        end
+      end
+
+      raise(UnRegisteredTypeError, name)
     end
 
     # Convert CamelCase +name+ to underscore style.
     def underscore_name(name)
-      name.to_s.sub(/.*::/, "").
-                gsub(/([A-Z]+)([A-Z][a-z])/, '\1_\2').
-                gsub(/([a-z\d])([A-Z])/, '\1_\2').
-                tr("-", "_").
-                downcase
+      name
+        .to_s
+        .sub(/.*::/, "")
+        .gsub(/([A-Z]+)([A-Z][a-z])/, '\1_\2')
+        .gsub(/([a-z\d])([A-Z])/, '\1_\2')
+        .tr('-', '_')
+        .downcase
     end
 
     #---------------
     private
 
-    def normalize_name(name, hints)
-      name = underscore_name(name)
+    def search_names(name, hints)
+      base = underscore_name(name)
+      searches = []
 
-      if !registered?(name)
-        search_prefix = [""].concat(Array(hints[:search_prefix]))
-        search_prefix.each do |prefix|
-          nwp = name_with_prefix(name, prefix)
-          if registered?(nwp)
-            name = nwp
-            break
-          end
+      search_prefix = [""] + Array(hints[:search_prefix])
+      search_prefix.each do |prefix|
+        nwp = name_with_prefix(base, prefix)
+        nwe = name_with_endian(nwp, hints[:endian])
 
-          nwe = name_with_endian(nwp, hints[:endian])
-          if registered?(nwe)
-            name = nwe
-            break
-          end
-        end
+        searches << nwp
+        searches << nwe if nwe
       end
 
-      name
+      searches
     end
 
     def name_with_prefix(name, prefix)
-      prefix = prefix.to_s.chomp("_")
+      prefix = prefix.to_s.chomp('_')
       if prefix == ""
         name
       else
@@ -86,26 +94,21 @@ module BinData
     end
 
     def name_with_endian(name, endian)
-      return name if endian.nil?
+      return nil if endian.nil?
 
-      suffix = (endian == :little) ? "le" : "be"
-      if /^u?int\d+$/ =~ name
+      suffix = (endian == :little) ? 'le' : 'be'
+      if /^u?int\d+$/.match?(name)
         name + suffix
       else
-        name + "_" + suffix
+        name + '_' + suffix
       end
     end
 
-    def registered?(name)
-      register_dynamic_class(name) unless @registry.key?(name)
-
-      @registry.key?(name)
-    end
-
     def register_dynamic_class(name)
-      if /^u?int\d+(le|be)$/ =~ name || /^s?bit\d+(le)?$/ =~ name
+      if /^u?int\d+(le|be)$/.match?(name) || /^s?bit\d+(le)?$/.match?(name)
         class_name = name.gsub(/(?:^|_)(.)/) { $1.upcase }
         begin
+          # call const_get for side effect of creating class
           BinData.const_get(class_name)
         rescue NameError
         end
@@ -114,9 +117,9 @@ module BinData
 
     def warn_if_name_is_already_registered(name, class_to_register)
       prev_class = @registry[name]
-      if $VERBOSE && prev_class && prev_class != class_to_register
-        warn "warning: replacing registered class #{prev_class} " \
-             "with #{class_to_register}"
+      if prev_class && prev_class != class_to_register
+        Kernel.warn "warning: replacing registered class #{prev_class} " \
+                    "with #{class_to_register}"
       end
     end
   end
